@@ -31,6 +31,8 @@
 #include <QFileInfo>
 #include <QStandardItemModel>
 #include <QThreadPool>
+#include <Plasma/Containment>
+#include <Plasma/Wallpaper>
 
 #include <KDebug>
 #include <KConfigDialog>
@@ -45,6 +47,8 @@
 #include <Plasma/DataEngine>
 #include <Plasma/ToolButton>
 #include <Plasma/Frame>
+#include <Plasma/ToolTipContent>
+#include <Plasma/ToolTipManager>
 
 #include <math.h>
 
@@ -97,6 +101,80 @@ void Frame::init()
     m_currentDay = QDate::currentDate();
     
     m_slideNumber = 0;
+
+    // Frame & Shadow dimensions
+    m_frameOutline = 8;
+    m_swOutline = 8;
+
+    // Initialize the slideshow timer
+    connect(m_mySlideShow, SIGNAL(pictureUpdated()), this, SLOT(scalePictureAndUpdate()));
+
+    connect(&m_waitForResize, SIGNAL(timeout()), this, SLOT(scalePictureAndUpdate()));
+    m_waitForResize.setSingleShot(true);
+    m_waitForResize.setInterval(200);
+    
+    configChanged();
+    
+    KConfigGroup cg = config();
+    if (frameReceivedUrlArgs) {
+        cg.writeEntry("url", m_currentUrl);
+        emit configNeedsSaving();
+    }
+
+    m_menuPresent = false;
+
+    QAction *openAction = action("run associated application");
+    openAction->setIcon(SmallIcon("image-x-generic"));
+    openAction->setText(i18n("&Open Picture..."));
+    QAction *wallpaperAction = new QAction(KIcon("user-desktop"),i18n("Set as Wallpaper Image"), this);
+    actions.append(wallpaperAction);
+    connect(wallpaperAction, SIGNAL(triggered(bool)), this, SLOT(setImageAsWallpaper()));
+}
+
+QList<QAction*> Frame::contextualActions()
+{
+    return actions;
+}
+
+void Frame::setImageAsWallpaper()
+{
+  //setting current image of Picture frame as wallaper image
+    KUrl url;
+
+    if (m_slideShow) {
+        url = m_mySlideShow->currentUrl();
+    } else {
+        url = m_currentUrl;
+    }
+
+    kDebug() << KMimeType::findByUrl(url).data()->name();
+
+    if (containment()->wallpaper() && containment()->wallpaper()->supportsMimetype(KMimeType::findByUrl(url).data()->name())) {
+       containment()->wallpaper()->setUrls(url);
+    } else {
+        KPluginInfo::List wallpaperList = containment()->wallpaper()->listWallpaperInfoForMimetype(KMimeType::findByUrl(url).data()->name());
+        bool image = false;
+        foreach (const KPluginInfo &wallpaper, wallpaperList) {
+            if (wallpaper.pluginName() == "image") {
+                image = true;
+                break;
+            }
+        }
+
+        if (image) {
+            containment()->setWallpaper("image");
+        } else if (!wallpaperList.isEmpty()) {
+            containment()->setWallpaper(wallpaperList.at(0).name());
+        }
+
+        if (containment()->wallpaper()) {
+            containment()->wallpaper()->setUrls(url);
+        }
+    }
+}
+
+void Frame::configChanged()
+{
     // Get config values
     KConfigGroup cg = config();
     m_frameColor = cg.readEntry("frameColor", QColor(70, 90, 130)); //theme?
@@ -114,28 +192,7 @@ void Frame::init()
     m_potd = cg.readEntry("potd", false);
     m_autoUpdateIntervall = cg.readEntry("autoupdate time", 0);
 
-    // Frame & Shadow dimensions
-    m_frameOutline = 8;
-    m_swOutline = 8;
-
-    // Initialize the slideshow timer
-    connect(m_mySlideShow, SIGNAL(pictureUpdated()), this, SLOT(scalePictureAndUpdate()));
-
-    connect(&m_waitForResize, SIGNAL(timeout()), this, SLOT(scalePictureAndUpdate()));
-    m_waitForResize.setSingleShot(true);
-    m_waitForResize.setInterval(200);
-
     initSlideShow();
-    if (frameReceivedUrlArgs) {
-        cg.writeEntry("url", m_currentUrl);
-        emit configNeedsSaving();
-    }
-
-    m_menuPresent = false;
-
-    QAction *openAction = action("run associated application");
-    openAction->setIcon(SmallIcon("image-x-generic"));
-    openAction->setText(i18n("&Open Picture..."));
 }
 
 void Frame::slotOpenPicture()
@@ -237,7 +294,7 @@ void Frame::scalePictureAndUpdate()
 {
     QImage img = m_mySlideShow->image();
     ImageScaler *scaler = new ImageScaler(img, contentSizeHint().toSize());
-    connect(scaler, SIGNAL(scaled(const QImage&)), this, SLOT(imageScaled(const QImage&)));
+    connect(scaler, SIGNAL(scaled(QImage)), this, SLOT(imageScaled(QImage)));
     QThreadPool::globalInstance()->start(scaler);
 }
 
@@ -266,6 +323,9 @@ void Frame::updatePicture()
     }
 
     kDebug() << "Rendering picture";
+    Plasma::ToolTipContent toolTipData;
+    toolTipData.setSubText(m_mySlideShow->currentUrl().fileName());
+    Plasma::ToolTipManager::self()->setContent(this, toolTipData);
 
     // create a QPixmap which can be drawn in paintInterface()
     QPixmap picture = QPixmap::fromImage(m_scaledImage);
@@ -386,12 +446,15 @@ void Frame::updatePicture()
         // Set the font and draw text
         p->setRenderHint(QPainter::Antialiasing);
         QFont textFont = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
-        textFont.setPixelSize(qMax(KGlobalSettings::smallestReadableFont().pixelSize(), bgRect.height() / 6));
+        textFont.setPointSize(qMax(KGlobalSettings::smallestReadableFont().pointSize(), bgRect.height() / 6));
         p->setFont(textFont);
 
         QTextOption option;
         option.setAlignment(Qt::AlignCenter);
         option.setWrapMode(QTextOption::WordWrap);
+
+        preparePainter(p, bgRect, textFont, message);
+
         p->setPen(QPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor), 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
         p->drawText(bgRect, message, option);
     }
@@ -406,6 +469,33 @@ void Frame::updatePicture()
     }
 }
 
+QRect Frame::preparePainter(QPainter *p, const QRect &rect, const QFont &font, const QString &text)
+{
+    QRect tmpRect;
+    QFont tmpFont = font;
+    bool first = true;
+
+    // Starting with the given font, decrease its size until it'll fit in the
+    // given rect allowing wrapping where possible
+    do {
+        if (first) {
+            first = false;
+        } else  {
+            tmpFont.setPointSize(qMax(KGlobalSettings::smallestReadableFont().pointSize(), tmpFont.pointSize() - 1));
+        }
+
+        const QFontMetrics fm(tmpFont);
+        int flags = Qt::TextWordWrap;
+
+        tmpRect = fm.boundingRect(rect, flags, text);
+    } while (tmpFont.pointSize() > KGlobalSettings::smallestReadableFont().pointSize() &&
+             (tmpRect.width() > rect.width() || tmpRect.height() > rect.height()));
+
+    p->setFont(tmpFont);
+    return tmpRect;
+}
+
+
 void Frame::nextPicture()
 {
     m_mySlideShow->setUpdateInterval(0);
@@ -479,7 +569,7 @@ void Frame::createConfigurationInterface(KConfigDialog *parent)
     connect(m_configDialog->imageUi.removeDirButton, SIGNAL(clicked()), this, SLOT(removeDir()));
     connect(m_configDialog->imageUi.addDirButton, SIGNAL(clicked()), this, SLOT(addDir()));
     connect(m_configDialog->imageUi.slideShowDirList, SIGNAL(currentRowChanged(int)), this, SLOT(updateButtons()));
-
+ 
     m_configDialog->setRoundCorners(m_roundCorners);
     m_configDialog->setShadow(m_shadow);
     m_configDialog->setShowFrame(m_frame);
@@ -508,6 +598,33 @@ void Frame::createConfigurationInterface(KConfigDialog *parent)
     m_configDialog->imageUi.slideShowDelay->setTime(QTime(m_slideshowTime / 3600, (m_slideshowTime / 60) % 60, m_slideshowTime % 60));
     m_configDialog->previewPicture(m_mySlideShow->image());
     m_configDialog->imageUi.autoUpdateTime->setTime(QTime(m_autoUpdateIntervall / 3600, (m_autoUpdateIntervall / 60) % 60));
+    
+    connect(m_configDialog->imageUi.slideShowDelay, SIGNAL(timeChanged(QTime)), 
+                   parent, SLOT(settingsModified()));
+    connect(m_configDialog->imageUi.pictureComboBox, SIGNAL(currentIndexChanged(int)), 
+                   parent, SLOT(settingsModified()));
+    connect(m_configDialog->imageUi.picRequester, SIGNAL(textChanged(QString)), 
+                   parent, SLOT(settingsModified()));
+    connect(m_configDialog->imageUi.autoUpdateTime, SIGNAL(timeChanged(QTime)), 
+                    parent, SLOT(settingsModified()));
+    connect(m_configDialog->imageUi.addDirButton, SIGNAL(clicked(bool)), 
+                   parent, SLOT(settingsModified()));
+    connect(m_configDialog->imageUi.removeDirButton, SIGNAL(clicked(bool)), 
+                   parent, SLOT(settingsModified()));
+    connect(m_configDialog->imageUi.randomCheckBox, SIGNAL(toggled(bool)),
+                   parent, SLOT(settingsModified()));  
+    connect(m_configDialog->imageUi.recursiveCheckBox, SIGNAL(toggled(bool)),
+                   parent, SLOT(settingsModified()));  
+    connect(m_configDialog->imageUi.potdComboBox, SIGNAL(currentIndexChanged(int)), 
+                   parent, SLOT(settingsModified()));
+    connect(m_configDialog->appearanceUi.roundCheckBox, SIGNAL(toggled(bool)),
+                   parent, SLOT(settingsModified()));
+    connect(m_configDialog->appearanceUi.shadowCheckBox, SIGNAL(toggled(bool)),
+                   parent, SLOT(settingsModified()));
+    connect(m_configDialog->appearanceUi.frameCheckBox, SIGNAL(toggled(bool)),
+                   parent, SLOT(settingsModified()));
+    connect(m_configDialog->appearanceUi.changeFrameColor,SIGNAL(changed(QColor)),
+                    parent, SLOT(settingsModified()));
 }
 
 void Frame::configAccepted()
@@ -581,8 +698,7 @@ void Frame::configAccepted()
 void Frame::stopPotd()
 {
     Plasma::DataEngine *engine = dataEngine("potd");
-    const QString identifier = m_potdProvider + ':' + m_currentDay.toString(Qt::ISODate);
-    engine->disconnectSource(identifier, m_mySlideShow);
+    engine->disconnectSource(m_potdProvider, m_mySlideShow);
     m_autoUpdateTimer->stop();
 }
 
@@ -596,12 +712,8 @@ void Frame::initSlideShow()
         m_mySlideShow->setDirs(m_slideShowPaths, m_recursiveSlideShow);
         m_mySlideShow->setUpdateInterval(m_slideshowTime * 1000);
     } else if (m_potd) {
-        m_dateChangedTimer = new QTimer( this );//change picture at midnight
-        connect( m_dateChangedTimer, SIGNAL( timeout() ), this, SLOT( checkDayChanged() ) );
-        m_dateChangedTimer->start( 60 * 60 * 1000 ); // every hour
         Plasma::DataEngine *engine = dataEngine("potd");
-        const QString identifier = m_potdProvider + ':' + m_currentDay.toString(Qt::ISODate);
-        engine->connectSource(identifier, m_mySlideShow);
+        engine->connectSource(m_potdProvider, m_mySlideShow);
     } else { //no slideshow so no random stuff
         m_mySlideShow->setRandom(false);
         m_mySlideShow->setImage(m_currentUrl.url());
@@ -610,7 +722,7 @@ void Frame::initSlideShow()
             m_doAutoUpdate = true;
         }
     }
-    
+
     scalePictureAndUpdate();
 }
 
@@ -629,11 +741,10 @@ void Frame::dropEvent(QGraphicsSceneDragDropEvent *event)
     if (m_slideFrame) {
         m_slideFrame->hide();
     }
-
     KUrl droppedUrl = (KUrl::List::fromMimeData(event->mimeData())).at(0);
     kDebug() << "dropped URL" << droppedUrl.url();
     if (droppedUrl.protocol() == "desktop") {
-        KUrl tmpUrl = KGlobalSettings::desktopPath() + droppedUrl.path();
+        KUrl tmpUrl = QString(KGlobalSettings::desktopPath() + droppedUrl.path());
         droppedUrl = tmpUrl;
     }
     // If the url is a local directory start slideshowmode
@@ -662,7 +773,7 @@ void Frame::dropEvent(QGraphicsSceneDragDropEvent *event)
 void Frame::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
 {
     checkSlideFrame();
-    if (m_slideShow) {    
+    if (m_slideShow) {
         m_slideFrame->show();
     }
     Applet::hoverEnterEvent(event);
@@ -735,28 +846,9 @@ void Frame::delayedUpdateSize()
     }
 }
 
-void Frame::checkDayChanged()
-{
-    if ( ( m_currentDay != QDate::currentDate() ) ) {
-        reloadImage();
-        //keep after reloadImage()
-        m_currentDay = QDate::currentDate();
-    }
-}
-
 void Frame::reloadImage()
 {
-    if (m_potd) {
-        Plasma::DataEngine *engine = dataEngine("potd");
-        //disconnect yesterday's source
-        QString identifier = m_potdProvider + ':' + m_currentDay.toString(Qt::ISODate);
-        engine->disconnectSource(identifier, m_mySlideShow);
-        //connect today's source
-        identifier = m_potdProvider + ':' + QDate::currentDate().toString(Qt::ISODate); 
-        engine->connectSource(identifier, m_mySlideShow);
-    } else {
-        m_mySlideShow->updateImage(m_currentUrl.url());
-    }
+    m_mySlideShow->updateImage(m_currentUrl.url());
 }
 
 #include "frame.moc"
