@@ -38,50 +38,37 @@
 #include "kimpanelinputpanel.h"
 
 KimpanelInputPanel::KimpanelInputPanel(QWidget* parent)
-    : QWidget(parent),
-      m_layout(new QHBoxLayout(this)),
-      m_scene(new QGraphicsScene(this)),
-      m_view(new QGraphicsView(m_scene, this)),
+    : QGraphicsView(parent),
       m_widget(new KimpanelInputPanelGraphics),
-      m_backgroundSvg(new Plasma::FrameSvg(this))
+      m_backgroundSvg(new Plasma::FrameSvg(this)),
+      m_composite(true),
+      m_useBlur(false)
 {
     setAttribute(Qt::WA_TranslucentBackground);
     setWindowFlags(Qt::ToolTip);
-    setAttribute(Qt::WA_X11DoNotAcceptFocus, true);
     KWindowSystem::setState(winId(), NET::KeepAbove);
     KWindowSystem::setType(winId(), NET::Tooltip);
-    QPalette pal = palette();
-    pal.setColor(backgroundRole(), Qt::transparent);
-    setPalette(pal);
 
-    m_layout->setContentsMargins(0, 0, 0, 0);
-    m_layout->setSpacing(0);
+    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    setFrameShape(QFrame::NoFrame);
+    setContentsMargins(0, 0, 0, 0);
+    setCacheMode(QGraphicsView::CacheBackground);
+    viewport()->setContentsMargins(0, 0, 0, 0);
 
-    //m_layout->setSizeConstraint(QLayout::SetFixedSize);
-    setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    setScene(new QGraphicsScene);
 
-    setLayout(m_layout);
-
-    m_view->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    m_view->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    m_view->setFrameShape(QFrame::NoFrame);
-    m_view->viewport()->setAutoFillBackground(false);
-    m_view->setContentsMargins(0, 0, 0, 0);
-    m_view->setAttribute(Qt::WA_NoSystemBackground);
-    m_view->viewport()->setAttribute(Qt::WA_NoSystemBackground);
-
-    m_layout->addWidget(m_view);
-
-    m_scene->addItem(m_widget);
-
-    loadTheme();
+    scene()->addItem(m_widget);
+    centerOn(m_widget);
 
     m_backgroundSvg->setCacheAllRenderedFrames(true);
+    m_backgroundSvg->setImagePath("dialogs/background");
+
+    loadTheme();
 
     Plasma::Theme* theme = Plasma::Theme::defaultTheme();
     connect(theme, SIGNAL(themeChanged()), this, SLOT(loadTheme()));
     connect(KWindowSystem::self(), SIGNAL(compositingChanged(bool)), this, SLOT(maskBackground(bool)));
-    connect(m_widget, SIGNAL(visibleChanged(bool)), this, SLOT(updateVisible(bool)));
     connect(m_widget, SIGNAL(sizeChanged()), this, SLOT(updateSize()));
 
     connect(m_widget, SIGNAL(selectCandidate(int)), this, SIGNAL(selectCandidate(int)));
@@ -92,6 +79,11 @@ KimpanelInputPanel::KimpanelInputPanel(QWidget* parent)
 void KimpanelInputPanel::loadTheme()
 {
     maskBackground(Plasma::Theme::defaultTheme()->windowTranslucencyEnabled());
+    QPalette plasmaPalette = QPalette();
+    plasmaPalette.setColor(QPalette::Window,
+                           Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor));
+    setAutoFillBackground(true);
+    setPalette(plasmaPalette);
 
     update();
 }
@@ -102,53 +94,58 @@ KimpanelInputPanel::~KimpanelInputPanel()
 
 void KimpanelInputPanel::showEvent(QShowEvent *e)
 {
-    QWidget::showEvent(e);
+    QGraphicsView::showEvent(e);
     Plasma::WindowEffects::overrideShadow(winId(), true);
     Display *dpy = QX11Info::display();
     Atom atom = XInternAtom( dpy, "_KDE_NET_WM_SHADOW", False );
     XDeleteProperty(dpy, winId(), atom);
+    updateLocation();
 }
 
 void KimpanelInputPanel::resizeEvent(QResizeEvent* event)
 {
-    QWidget::resizeEvent(event);
-
-    QRectF rect(QPointF(0, 0), size());
-    m_scene->setSceneRect(rect);
-    m_backgroundSvg->resizeFrame(event->size());
+    QGraphicsView::resizeEvent(event);
     updateLocation();
 
     maskBackground(Plasma::Theme::defaultTheme()->windowTranslucencyEnabled());
-    update();
+    resetCachedContent();
 }
 
 void KimpanelInputPanel::maskBackground(bool composite)
 {
-    if (!composite) {
-        m_backgroundSvg->setImagePath("opaque/dialogs/background");
+    if (m_composite != composite) {
+        m_composite = composite;
+        m_backgroundSvg->clearCache();
+        if (!m_composite) {
+            m_backgroundSvg->setImagePath("opaque/dialogs/background");
+        } else {
+            m_backgroundSvg->setImagePath("dialogs/background");
+            clearMask();
+        }
+    }
+    m_backgroundSvg->resizeFrame(size());
+    if (!m_composite) {
         setMask(m_backgroundSvg->mask());
-    } else {
-        m_backgroundSvg->setImagePath("dialogs/background");
-        clearMask();
     }
 
-    if (Plasma::WindowEffects::isEffectAvailable(Plasma::WindowEffects::BlurBehind) && KimpanelSettings::self()->enableBackgroundBlur()) {
+    bool useBlur = Plasma::WindowEffects::isEffectAvailable(Plasma::WindowEffects::BlurBehind) && KimpanelSettings::self()->enableBackgroundBlur();
+    if (m_useBlur != useBlur) {
+        m_useBlur = useBlur;
+        if (!m_useBlur) {
+            Plasma::WindowEffects::enableBlurBehind(winId(), false);
+        }
+    }
+    if (m_useBlur) {
         Plasma::WindowEffects::enableBlurBehind(winId(), true, m_backgroundSvg->mask());
-    } else {
-        Plasma::WindowEffects::enableBlurBehind(winId(), false);
     }
-    qreal left, right, top, bottom;
-    m_backgroundSvg->getMargins(left, top, right, bottom);
-    setContentsMargins(left, top, right, bottom);
 }
 
-void KimpanelInputPanel::paintEvent(QPaintEvent *e)
+void KimpanelInputPanel::drawBackground(QPainter* painter, const QRectF& rect)
 {
-    QPainter p(this);
-    p.setClipRect(e->rect());
-    p.setCompositionMode(QPainter::CompositionMode_Source);
-    p.fillRect(e->rect(), Qt::transparent);
-    m_backgroundSvg->paintFrame(&p);
+    painter->setClipRect(rect);
+    painter->setCompositionMode(QPainter::CompositionMode_Source);
+    painter->fillRect(rect, Qt::transparent);
+    m_backgroundSvg->paintFrame(painter);
 }
 
 void KimpanelInputPanel::setSpotLocation(const QRect& rect)
@@ -231,19 +228,25 @@ void KimpanelInputPanel::updateVisible(bool visible)
     setVisible(visible);
 }
 
-void KimpanelInputPanel::updateSize()
+void KimpanelInputPanel::updateSizeVisibility()
 {
-    int left, top, right, bottom;
-    getContentsMargins(&left, &top, &right, &bottom);
-    QSize sizeHint = m_widget->size().toSize();
-    sizeHint += QSize(left + right, top + bottom);
-    setMinimumSize(sizeHint);
-    setMaximumSize(sizeHint);
-    if (m_view->size() != m_widget->preferredSize().toSize()) {
-        m_view->resize(m_widget->preferredSize().toSize());
-        m_scene->setSceneRect(QRectF(QPointF(0, 0), (m_widget->preferredSize())));
-        updateLocation();
-        m_backgroundSvg->resizeFrame(sizeHint);
+    m_widget->updateVisible();
+    bool newVisible = m_widget->markedVisible();
+    if (newVisible) {
+        m_widget->updateSize();
     }
+    updateVisible(newVisible);
 }
 
+void KimpanelInputPanel::updateSize()
+{
+    qreal left, top, right, bottom;
+    m_backgroundSvg->getMargins(left, top, right, bottom);
+    QSize size = m_widget->roundSize();
+    m_widget->setPos(left, top);
+    QSize sizeHint = size + QSize(left + right, top + bottom);
+    setSceneRect(0, 0, sizeHint.width(), sizeHint.height());
+    resize(sizeHint);
+}
+
+
