@@ -1,5 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2007 by Thomas Georgiou <TAGeorgiou@gmail.com>          *
+ *                         Artur Duque de Souza <asouza@kde.org>           *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -21,7 +22,6 @@
 #define PASTEBIN_H
 
 #include "ui_pastebinConfig.h"
-#include "ui_pastebinServersConfig.h"
 
 #include <KDE/KIO/TransferJob>
 #include <KDE/KIO/Job>
@@ -44,7 +44,11 @@ class QPropertyAnimation;
 namespace Plasma
 {
     class Service;
-} // namespace Plasma
+}
+
+namespace KNS3 {
+    class DownloadDialog;
+}
 
 class Pastebin : public Plasma::Applet
 {
@@ -85,7 +89,10 @@ public:
 
 public slots:
     void configAccepted();
-    void dataUpdated(const QString &sourceName, const Plasma::DataEngine::Data &data);
+    void configChanged();
+    void dataUpdated(const QString &source, const Plasma::DataEngine::Data &data);
+    void sourceAdded(const QString &source);
+    void sourceRemoved(const QString &source);
 
 protected slots:
     virtual void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
@@ -111,18 +118,15 @@ private slots:
     void updateTheme();
     void resetActionState();
     void copyToClipboard(const QString &url);
-    void postingFinished(Plasma::ServiceJob *job);
-
-    void editTextServer();
-    void saveTextServer();
-    void editImageServer();
-    void saveImageServer();
-    void closeServerDialog();
+    void postingFinished(KJob *job);
+    void getNewStuff();
+    void newStuffFinished();
+    void refreshConfigDialog();
 
 private:
     int iconSize();
     void showOverlay(bool show);
-    void postContent(QString text, QImage imageData);
+    void postContent(QString text, const QImage& imageData);
     void postClipboard(bool preferSelection);
 
     void addToHistory(const QString &url);
@@ -135,9 +139,11 @@ private:
 
     bool m_isHovered;
     bool m_fadeIn;
-    QWeakPointer<QPropertyAnimation> m_animation;
     qreal m_alpha;
 
+    QTimer *m_timer;
+    QWeakPointer<QPropertyAnimation> m_animation;
+
     QFont m_font;
     QPen m_linePen;
     QColor m_fgColor;
@@ -147,14 +153,6 @@ private:
     QString m_url;
     QString m_oldUrl;
 
-    int m_textBackend;
-    int m_imageBackend;
-    int m_imagePrivacy;
-    QString m_server;
-
-    QTimer *m_timer;
-
-    int m_historySize;
     QSignalMapper *m_signalMapper;
     QList<QAction*> m_contextualActions;
     QList<QAction*> m_actionHistory;
@@ -162,17 +160,17 @@ private:
     QAction *m_topSeparator;
     QAction *m_bottomSeparator;
 
-    // custom server dialog
-    QWidget *m_servers;
+    // New version - below here vars after refactor
+    int m_historySize;
+
+    QHash<QString, QString> m_txtServers;
+    QHash<QString, QString> m_imgServers;
+    Plasma::DataEngine *m_engine;
+    Plasma::Service *m_postingService;
+    Ui::pastebinConfig uiConfig;
+    KNS3::DownloadDialog* m_newStuffDialog;
 
     QClipboard::Mode lastMode;
-
-    Ui::pastebinConfig uiConfig;
-    Ui::pastebinServersConfig uiServers;
-
-    Plasma::Service *m_postingService;
-
-    QString m_hoverText;
 };
 
 K_EXPORT_PLASMA_APPLET(pastebin, Pastebin)
