@@ -1,11 +1,11 @@
 /*
  *   Copyright (C) 2007 Tobias Koenig <tokoe@kde.org>
- *   Copyright  2008 by Anne-Marie Mahfouf <annma@kde.org>                 
+ *   Copyright  2008 by Anne-Marie Mahfouf <annma@kde.org>
  *
- *   This program is free software; you can redistribute it and/or modify  
- *   it under the terms of the GNU General Public License as published by  
- *   the Free Software Foundation; either version 2 of the License, or     
- *   (at your option) any later version.   
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +20,6 @@
 
 #include "wcpotdprovider.h"
 
-#include <QtCore/QDate>
 #include <QtCore/QRegExp>
 #include <QtGui/QImage>
 
@@ -43,7 +42,6 @@ class WcpotdProvider::Private
 
     WcpotdProvider *mParent;
     QByteArray mPage;
-    QDate mDate;
     QImage mImage;
 };
 
@@ -55,10 +53,8 @@ void WcpotdProvider::Private::pageRequestFinished( KJob *_job )
 	return;
     }
 
-
-    KUrl picUrl( QString( "http://tools.wikimedia.de/~daniel/potd/potd.php/commons/800x600" ) );
-    KIO::StoredTransferJob *imageJob = KIO::storedGet( picUrl );
-    mParent->connect( imageJob, SIGNAL( finished( KJob *) ), SLOT( imageRequestFinished( KJob* ) ) );
+    KUrl picUrl( QLatin1String( "http://tools.wikimedia.de/~daniel/potd/potd.php/commons/800x600" ) );  KIO::StoredTransferJob *imageJob = KIO::storedGet( picUrl, KIO::NoReload, KIO::HideProgressInfo );
+    mParent->connect( imageJob, SIGNAL(finished(KJob*)), SLOT(imageRequestFinished(KJob*)) );
 }
 
 void WcpotdProvider::Private::imageRequestFinished( KJob *_job )
@@ -76,15 +72,10 @@ void WcpotdProvider::Private::imageRequestFinished( KJob *_job )
 WcpotdProvider::WcpotdProvider( QObject *parent, const QVariantList &args )
     : PotdProvider( parent, args ), d( new Private( this ) )
 {
-    const QString type = args[ 0 ].toString();
-    if ( type == "Date" )
-        d->mDate = args[ 1 ].toDate();
-    else
-	Q_ASSERT( false && "Invalid type passed to potd provider" );
+    KUrl url( QLatin1String( "http://tools.wikimedia.de/~daniel/potd/potd.php/commons/800x600" ));
 
-    KUrl url( "http://tools.wikimedia.de/~daniel/potd/potd.php/commons/800x600" );
-    KIO::StoredTransferJob *job = KIO::storedGet( url );
-    connect( job, SIGNAL( finished( KJob *) ), SLOT( pageRequestFinished( KJob* ) ) );
+    KIO::StoredTransferJob *job = KIO::storedGet( url, KIO::NoReload, KIO::HideProgressInfo );
+    connect( job, SIGNAL(finished(KJob*)), SLOT(pageRequestFinished(KJob*)) );
 }
 
 WcpotdProvider::~WcpotdProvider()
@@ -97,9 +88,4 @@ QImage WcpotdProvider::image() const
     return d->mImage;
 }
 
-QString WcpotdProvider::identifier() const
-{
-    return QString( "wcpotd:%1" ).arg( d->mDate.toString( Qt::ISODate ));
-}
-
 #include "wcpotdprovider.moc"
